#ifndef LINE_CUBIC_TRANSFORMATION_HPP
#define LINE_CUBIC_TRANSFORMATION_HPP

#include <algebra/TinyVector.hpp>

template <size_t GivenDimension>
class LineCubicTransformation
{
 public:
  constexpr static size_t Dimension = GivenDimension;

 private:
  TinyVector<Dimension> m_alpha;
  TinyVector<Dimension> m_beta;
  TinyVector<Dimension> m_gamma;
  TinyVector<Dimension> m_delta;

 public:
  PUGS_INLINE
  TinyVector<Dimension>
  operator()(const TinyVector<1>& x) const
  {
    return x[0] * (x[0] * (x[0] * m_alpha + m_beta) + m_gamma) + m_delta;
  }

  PUGS_INLINE
  TinyVector<Dimension>
  velocity(const TinyVector<1>& x) const
  {
    return x[0] * (3 * x[0] * m_alpha + 2 * m_beta) + m_gamma;
  }

  PUGS_INLINE
  double
  velocityNorm(const TinyVector<1>& x) const
  {
    return l2Norm(velocity(x));
  }

  PUGS_INLINE
  LineCubicTransformation(const TinyVector<Dimension>& a0,
                          const TinyVector<Dimension>& a1,
                          const TinyVector<Dimension>& a2,
                          const TinyVector<Dimension>& a3)
    : m_alpha{0.0625 * (-9 * a0 + 27 * a1 - 27 * a2 + 9 * a3)},
      m_beta{0.0625 * (9 * a0 - 9 * a1 - 9 * a2 + 9 * a3)},
      m_gamma{0.0625 * (a0 - 27 * a1 + 27 * a2 - a3)},
      m_delta{0.0625 * (-a0 + 9 * a1 + 9 * a2 - a3)}
  {
    static_assert(Dimension > 1);
  }

  ~LineCubicTransformation() = default;
};

#endif   // LINE_CUBIC_TRANSFORMATION_HPP
