#ifndef LINE_PARABOLIC_TRANSFORMATION_HPP
#define LINE_PARABOLIC_TRANSFORMATION_HPP

#include <algebra/TinyVector.hpp>

template <size_t GivenDimension>
class LineParabolicTransformation
{
 public:
  constexpr static size_t Dimension = GivenDimension;

 private:
  TinyVector<Dimension> m_alpha;
  TinyVector<Dimension> m_beta;
  TinyVector<Dimension> m_gamma;

 public:
  PUGS_INLINE
  TinyVector<Dimension>
  operator()(const TinyVector<1>& x) const
  {
    return x[0] * (x[0] * m_alpha + m_beta) + m_gamma;
  }

  PUGS_INLINE
  TinyVector<Dimension>
  velocity(const TinyVector<1>& x) const
  {
    return (2 * x[0]) * m_alpha + m_beta;
  }

  PUGS_INLINE
  double
  velocityNorm(const TinyVector<1>& x) const
  {
    return l2Norm(velocity(x));
  }

  PUGS_INLINE
  LineParabolicTransformation(const TinyVector<Dimension>& a0,
                              const TinyVector<Dimension>& a1,
                              const TinyVector<Dimension>& a2)
    : m_alpha{0.5 * (a0 - 2 * a1 + a2)}, m_beta{0.5 * (a2 - a0)}, m_gamma{a1}
  {
    static_assert(Dimension > 1);
  }

  ~LineParabolicTransformation() = default;
};

#endif   // LINE_PARABOLIC_TRANSFORMATION_HPP
