#include <mesh/MeshVariant.hpp>

#include <mesh/Connectivity.hpp>
#include <mesh/Mesh.hpp>
#include <mesh/PolynomialMesh.hpp>

size_t
MeshVariant::id() const
{
  return std::visit([](auto&& mesh) { return mesh->id(); }, m_p_mesh_variant);
}

std::ostream&
operator<<(std::ostream& os, const MeshVariant& mesh_v)
{
  std::visit([&](auto&& p_mesh) { os << *p_mesh; }, mesh_v.variant());
  return os;
}

size_t
MeshVariant::dimension() const
{
  return std::visit([](auto&& mesh) { return mesh->dimension(); }, m_p_mesh_variant);
}

size_t
MeshVariant::numberOfCells() const
{
  return std::visit([](auto&& mesh) { return mesh->numberOfCells(); }, m_p_mesh_variant);
}

size_t
MeshVariant::numberOfFaces() const
{
  return std::visit([](auto&& mesh) { return mesh->numberOfFaces(); }, m_p_mesh_variant);
}

size_t
MeshVariant::numberOfEdges() const
{
  return std::visit([](auto&& mesh) { return mesh->numberOfEdges(); }, m_p_mesh_variant);
}

size_t
MeshVariant::numberOfNodes() const
{
  return std::visit([](auto&& mesh) { return mesh->numberOfNodes(); }, m_p_mesh_variant);
}

const IConnectivity&
MeshVariant::connectivity() const
{
  return std::visit([](auto&& mesh) -> const IConnectivity& { return mesh->connectivity(); }, m_p_mesh_variant);
}

std::shared_ptr<const IConnectivity>
MeshVariant::shared_connectivity() const
{
  return std::visit([](auto&& mesh) -> std::shared_ptr<const IConnectivity> { return mesh->shared_connectivity(); },
                    m_p_mesh_variant);
}

template <MeshConcept MeshType>
std::shared_ptr<const MeshType>
MeshVariant::_get() const
{
  if (not std::holds_alternative<std::shared_ptr<const MeshType>>(this->m_p_mesh_variant)) {
    std::ostringstream error_msg;
    error_msg << "invalid mesh type type\n";
    error_msg << "- required " << rang::fgB::red << demangle<MeshType>() << rang::fg::reset << '\n';
    error_msg << "- contains " << rang::fgB::yellow
              << std::visit([](auto&& p_mesh) -> std::string { return demangle<mesh_type_t<decltype(p_mesh)>>(); },
                            this->m_p_mesh_variant)
              << rang::fg::reset;
    throw NormalError(error_msg.str());
  }
  return std::get<std::shared_ptr<const MeshType>>(m_p_mesh_variant);
}

template std::shared_ptr<const Mesh<1>> MeshVariant::_get<Mesh<1>>() const;
template std::shared_ptr<const Mesh<2>> MeshVariant::_get<Mesh<2>>() const;
template std::shared_ptr<const Mesh<3>> MeshVariant::_get<Mesh<3>>() const;

template std::shared_ptr<const PolynomialMesh<2>> MeshVariant::_get<PolynomialMesh<2>>() const;
