#ifndef MESH_VARIANT_HPP
#define MESH_VARIANT_HPP

#include <mesh/MeshTraits.hpp>
#include <utils/Demangle.hpp>
#include <utils/Exceptions.hpp>
#include <utils/PugsMacros.hpp>

#include <rang.hpp>

#include <iostream>
#include <memory>
#include <sstream>
#include <variant>

class IConnectivity;

template <size_t Dimension>
class Mesh;

template <size_t Dimension>
class PolynomialMesh;

class MeshVariant
{
 private:
  using Variant = std::variant<std::shared_ptr<const Mesh<1>>,   //
                               std::shared_ptr<const Mesh<2>>,   //
                               std::shared_ptr<const Mesh<3>>,   //

                               std::shared_ptr<const PolynomialMesh<2>>>;

  Variant m_p_mesh_variant;

  template <MeshConcept MeshType>
  std::shared_ptr<const MeshType> _get() const;

 public:
  friend std::ostream& operator<<(std::ostream& os, const MeshVariant& mesh_v);

  size_t id() const;

  size_t dimension() const;
  size_t numberOfCells() const;
  size_t numberOfFaces() const;
  size_t numberOfEdges() const;
  size_t numberOfNodes() const;

  const IConnectivity& connectivity() const;
  std::shared_ptr<const IConnectivity> shared_connectivity() const;

  template <MeshConcept MeshType>
  PUGS_INLINE std::shared_ptr<const MeshType>
  get() const
  {
    return this->_get<std::remove_const_t<MeshType>>();
  }

  PUGS_INLINE
  Variant
  variant() const
  {
    return m_p_mesh_variant;
  }

  MeshVariant() = delete;

  template <MeshConcept MeshType>
  MeshVariant(const std::shared_ptr<const MeshType>& p_mesh) : m_p_mesh_variant{p_mesh}
  {}

  MeshVariant(const MeshVariant&) = default;
  MeshVariant(MeshVariant&&)      = default;

  ~MeshVariant() = default;
};

#endif   // MESH_VARIANT_HPP
