#ifndef GLOBAL_VARIABLE_MANAGER_HPP
#define GLOBAL_VARIABLE_MANAGER_HPP

#include <utils/Exceptions.hpp>
#include <utils/PugsAssert.hpp>
#include <utils/PugsMacros.hpp>

#include <optional>

class GlobalVariableManager
{
 private:
  // Give some special access for testing
  friend class NbGhostLayersTester;

  size_t m_connectivity_id = 0;
  size_t m_mesh_id         = 0;

  std::optional<size_t> m_number_of_ghost_layers;

  static GlobalVariableManager* m_instance;

  explicit GlobalVariableManager()                    = default;
  GlobalVariableManager(GlobalVariableManager&&)      = delete;
  GlobalVariableManager(const GlobalVariableManager&) = delete;
  ~GlobalVariableManager()                            = default;

 public:
  PUGS_INLINE
  size_t
  getConnectivityId() const
  {
    return m_connectivity_id;
  }

  PUGS_INLINE
  size_t
  getAndIncrementConnectivityId()
  {
    return m_connectivity_id++;
  }

  PUGS_INLINE
  void
  setConnectivityId(size_t connectivity_id)
  {
    m_connectivity_id = connectivity_id;
  }

  PUGS_INLINE
  size_t
  getMeshId() const
  {
    return m_mesh_id;
  }

  PUGS_INLINE
  size_t
  getAndIncrementMeshId()
  {
    return m_mesh_id++;
  }

  PUGS_INLINE
  void
  setNumberOfGhostLayers(const size_t number)
  {
    if (m_number_of_ghost_layers.has_value()) {
      throw UnexpectedError("changing number of ghost layers is forbidden");
    }
    m_number_of_ghost_layers = number;
  }

  PUGS_INLINE
  size_t
  getNumberOfGhostLayers()
  {
    Assert(m_number_of_ghost_layers.has_value());
    return m_number_of_ghost_layers.value();
  }

  PUGS_INLINE
  void
  setMeshId(size_t mesh_id)
  {
    m_mesh_id = mesh_id;
  }

  PUGS_INLINE
  static GlobalVariableManager&
  instance()
  {
    Assert(m_instance != nullptr);
    return *m_instance;
  }

  static void create();
  static void destroy();
};

#endif   // GLOBAL_VARIABLE_MANAGER_HPP
